﻿# grafikfenster.py
# Auf der Grundlage der wxPython Demo
# kommentierte Version ohne Singleton-Exception

import wx
import colorsys
from math import radians

USE_BUFFER = ('wxMSW' in wx.PlatformInfo) # use buffered drawing on Windows

### ---------------------------------------------------
class Zeichenflaeche(wx.Panel):
    """Die Klasse stellt eine Zeichenflaeche fuer Vektorgrafik bereit.
    Sie wendet das Singleton-Muster an, also kein Konstruktoraufruf,
    sondern Methode GibZeichenflaeche()"""
    __zeichenflaeche=None
    @staticmethod
    def GibZeichenflaeche(parent=None):
        if Zeichenflaeche.__zeichenflaeche == None:
            if parent==None:
                return None
            else:
                Zeichenflaeche.__zeichenflaeche=Zeichenflaeche(parent)
        return Zeichenflaeche.__zeichenflaeche

    def __init__(self, parent):
        """Konstruktor
        !nicht direkt aufrufen!"""
        wx.Panel.__init__(self, parent, -1)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        if USE_BUFFER:
            self.Bind(wx.EVT_SIZE, self.OnSize)
        self.objekte = []

    def OnSize(self, evt):
        """Ereignismethode, die beim Aendern der Groesse
        des fensters aufgerufen wird."""
        self.InitBuffer()
        evt.Skip()

    def OnPaint(self, evt):
        """Ereignismethode fuer das Zeichnen
        Die erste Anweisung ist im Unterschied zur Demo notwendig,
        um unter Windows beim ersten Zeichnen
        den Puffer richtig zu initialisieren"""
        self.InitBuffer()
        if USE_BUFFER:
            dc = wx.BufferedPaintDC(self, self._buffer)
        else:
            dc = wx.PaintDC(self)
            self.gc = self.MakeGC(dc)
            self.Draw(self.gc)

    def InitBuffer(self):
        """Initialisierungsmethode fuer den Zeichenpuffer"""
        sz = self.GetClientSize()
        sz.width = max(1, sz.width)
        sz.height = max(1, sz.height)
        self._buffer = wx.EmptyBitmap(sz.width, sz.height, 32)
        dc = wx.MemoryDC(self._buffer)
        dc.SetBackground(wx.Brush("WHITE"))
        dc.Clear()
        self.gc = self.MakeGC(dc)
        self.Draw(self.gc)

    def MakeGC(self, dc):
        """erzeugt den 'Graphics-Context'"""
        try:
            gc = wx.GraphicsContext.Create(dc)
        except NotImplementedError:
            dc.DrawText("This build of wxPython does not support the wx.GraphicsContext "
                        "family of classes.",
                        25, 25)
            return None
        self.gc = gc
        return gc

    def Draw(self, gc):
        """Die eigentliche Zeichenmethode
        wxPython transformiert den Zeichenpfad.
        Wenn man mehrfach Zeichnen will, muss man daher
        vor einer Transformation den Zustand sichern und
        anschliessend wiederherstellen."""
        font = wx.SystemSettings.GetFont(wx.SYS_DEFAULT_GUI_FONT)
        font.SetWeight(wx.BOLD)
        gc.SetFont(font)

        # weissen Hintergrund zeichnen
        back = gc.CreatePath()
        gc.PushState()  # save current translation/scale/other state
        back.AddRectangle(0, 0, self.GetClientSize().width, self.GetClientSize().height)
        gc.SetBrush(wx.Brush('white'))
        gc.FillPath(back)          # nur Fuellung
        gc.PopState()              # restore saved state

        for objekt in self.objekte:
            if objekt.GibSichtbar():
                path = gc.CreatePath()
                gc.PushState()
                path.AddPath(objekt.GibFigur())
                gc.SetPen(wx.Pen(objekt.GibFarbe()))
                gc.DrawPath(path)
                gc.PopState()

    def Zeichne(self, objekt):
        """Erweiterungen für das Raumplanerprojekt:
        Ruft nacheinander das Zeichnen fuer alle Objekte auf."""
        if objekt in self.objekte:
            self.objekte.remove(objekt)
        self.objekte.append(objekt)
        self.Refresh()

    def Entferne(self, objekt):
        """Entfernt ein Objekt aus der Darstellung."""
        if objekt in self.objekte: self.objekte.remove(objekt)
        self.Refresh()

    def GibGC(self):
        """Gibt den aktuellen Graphics-Context zurueck."""
        return self.gc

### ---------------------------------------------------
from wx.py.shell import ShellFrame
from wx.py.filling import FillingFrame
### ---------------------------------------------------
class GrafikFenster(wx.Frame):
    """
    Die Klasse GrafikFenster erzeugt einen Frame.
    """
    def __init__(self, parent, title):
        wx.Frame.__init__(self, parent, -1, title,
                          pos=(20, 50), size=(400, 400))
        if wx.PlatformInfo[1]=='wxMSW':
            self.SetSize((400,420))
        self.panel = Zeichenflaeche.GibZeichenflaeche(self)
        self.shellFrame = ShellFrame(parent=self, pos=wx.Point(450,100))
        self.fillingFrame = FillingFrame(parent=self, pos=wx.Point(40,500))

    def ZeigeShellFrame(self, zeigen=True):
        """Zeigt die interaktive Shell an."""
        self.shellFrame.Show(zeigen)

    def ZeigeFillingFrame(self, zeigen=True):
        """Zeigt den Monitor an."""
        self.fillingFrame.Show(zeigen)

### ---------------------------------------------------
class Stuhl():
    """Klasse Stuhl
    ermoeglicht das Zeichnen und Bearbeiten eines
    Stuhl-Symbols fuer den Raumplaner"""
    
    def __init__(self,
                 xPos=20,
                 yPos=20,
                 breite=40,
                 tiefe=40,
                 winkel=0,
                 farbe="blue",
                 sichtbar=False):
        """Konstruktor mit vordefinierten Parametern"""
        self.x=xPos
        self.y=yPos
        self.b=breite
        self.t=tiefe
        self.w=winkel
        self.f=farbe
        self.s=sichtbar
        if sichtbar: self.Zeige()
        
    def GibFigur(self):
        """definiert und transformiert die zu zeichnende Figur"""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        path = gc.CreatePath()
        
        path.MoveToPoint(0, 0)
        path.AddLineToPoint(self.b, 0)
        path.AddLineToPoint(self.b*1.1, self.t)
        path.AddLineToPoint(-self.b*0.1, self.t)
        path.AddLineToPoint(0, 0)
        path.AddLineToPoint(0, -self.t*0.1)
        path.AddLineToPoint(self.b, -self.t*0.1)
        path.AddLineToPoint(self.b, 0)
        
        gc.PushState()
        gc.Translate(self.x+self.b/2, self.y+self.t/2)
        gc.Rotate(radians(self.w))
        gc.Translate(-self.b/2, -self.t/2)
        transformation = gc.GetTransform()
        gc.PopState()
        path.Transform(transformation)
        return path

    def GibFarbe(self):
        """Get-Methode fuer die Farbe"""
        return self.f
    
    def GibSichtbar(self):
        """Get-Methode fuer die Sichtbarkeit"""
        return self.s

    def BewegeHorizontal(self, weite):
        """Veraendernde Methode fuer die x-Position"""
        self.Verberge()
        self.x += weite
        self.Zeige()
    
    def BewegeVertikal(self, weite):
        """Veraendernde Methode fuer die y-Position"""
        self.Verberge()
        self.y += weite
        self.Zeige()
    
    def Drehe(self, winkel):
        """Veraendernde Methode fuer die Orientierung [Winkel]"""
        self.Verberge()
        self.w += winkel
        self.Zeige()

    def Verberge(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert False"""
        self.s = False
        Zeichenflaeche.GibZeichenflaeche().Entferne(self)

    def Zeige(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert True"""
        self.s = True
        Zeichenflaeche.GibZeichenflaeche().Zeichne(self)

### ---------------------------------------------------
class Tisch():
    """Klasse Tisch
    ermoeglicht das Zeichnen und Bearbeiten eines
    Tisch-Symbols fuer den Raumplaner"""

    def __init__(self,
                 xPos=60,
                 yPos=10,
                 breite=120,
                 tiefe=60,
                 winkel=0,
                 farbe='red',
                 sichtbar=False):
        """Konstruktor mit vordefinierten Parametern"""
        self.x=xPos
        self.y=yPos
        self.b=breite
        self.t=tiefe
        self.w=winkel
        self.f=farbe
        self.s=sichtbar
        if sichtbar: self.Zeige()
        
    def GibFigur(self):
        """definiert und transformiert die zu zeichnende Figur"""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        path = gc.CreatePath()
        
        path.AddRectangle(0, 0, self.b, self.t)
        
        gc.PushState()
        gc.Translate(self.x+self.b/2, self.y+self.t/2)
        gc.Rotate(radians(self.w))
        gc.Translate(-self.b/2, -self.t/2)
        transformation = gc.GetTransform()
        gc.PopState()
        path.Transform(transformation)
        return path

    def GibFarbe(self):
        """Get-Methode fuer die Farbe"""
        return self.f
    
    def GibSichtbar(self):
        """Get-Methode fuer die Sichtbarkeit"""
        return self.s

    def BewegeHorizontal(self, weite):
        """Veraendernde Methode fuer die x-Position"""
        self.Verberge()
        self.x += weite
        self.Zeige()
    
    def BewegeVertikal(self, weite):
        """Veraendernde Methode fuer die y-Position"""
        self.Verberge()
        self.y += weite
        self.Zeige()
    
    def Drehe(self, winkel):
        """Veraendernde Methode fuer die Orientierung [Winkel]"""
        self.Verberge()
        self.w += winkel
        self.Zeige()

    def Verberge(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert False"""
        self.s = False
        Zeichenflaeche.GibZeichenflaeche().Entferne(self)

    def Zeige(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert True"""
        self.s = True
        Zeichenflaeche.GibZeichenflaeche().Zeichne(self)

### ---------------------------------------------------
class MyApp(wx.App):
    """Testanwendung fuer Raumplaner-Grafik
       ShellFrame: Interaktion mit laufender Anwendung
       FillingFrame: Anzeige der Umgebung"""
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        #self.fenster.ZeigeFillingFrame()
        self.TestAnwendung()
        return True

    def TestAnwendung(self):
        """Allein fuer die Testanwendung:"""
        global tisch, stuhl
        tisch=Tisch(70, 30, 120, 60, 0, 'red', True)
        stuhl=Stuhl(20, 40, 40, 40, 270, 'blue', True)

### ---------------------------------------------------
if __name__ == '__main__':
    app = MyApp(redirect=False)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
